package broker

import (
	"math/rand"
	"time"

	"gitlab.33.cn/link33/chain33-sdk-go/crypto"
	"gitlab.33.cn/link33/chain33-sdk-go/types"
)

// 构造发布跨链交易事件交易
func EmitInterchainEvent(paraName, dstServiceID, funcName string, args []string, txType uint64) (*types.Transaction, error) {
	//目的链ID,固定格式0x000001
	//string  dstServiceID = 2;
	////源链ID,固定格式0x000002
	//string  srcServiceID = 3;
	////跨链交易类型 0表示storage,1表示coins,2表示token......
	//uint64  type =4;
	////调用方法
	//string  func = 5;
	////参数列表
	//repeated string  args = 6;
	payload := &types.BrokerAction{Ty: TyEmitInterchainEventAction, Value: &types.BrokerAction_EmitInterchainEvent{EmitInterchainEvent: &types.InterchainEvent{DstServiceID: dstServiceID, Type: txType, Func: funcName, Args: args}}}
	if paraName == "" {
		tx := &types.Transaction{Execer: []byte(BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: Addr}
		return tx, nil
	} else {
		tx := &types.Transaction{Execer: []byte(paraName + BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: crypto.GetExecAddress(paraName + BrokerX)}
		return tx, nil
	}
}

// 构造更新Index交易
func UpdateIndex(paraName, srcServiceID, dstServiceID string, sequenceNum, reqType uint64, response *types.Response) (*types.Transaction, error) {
	////当前链已经处理到的位置
	//uint64  sequenceNum = 1;
	////目的链服务ID,固定格式0x000001
	//string  dstServiceID = 2;
	////源链ID,固定格式0x000002
	//string  srcServiceID = 3;
	////请求类型 0表示信息流入 1表示信息流出 2表示回滚
	//uint64  reqType =4;
	////响应信息
	//Response   response = 5;
	payload := &types.BrokerAction{Ty: TyUpdateIndexAction, Value: &types.BrokerAction_UpdateIndex{UpdateIndex: &types.UpdateIndex{DstServiceID: dstServiceID, SrcServiceID: srcServiceID, ReqType: reqType, SequenceNum: sequenceNum, Response: response}}}
	if paraName == "" {
		tx := &types.Transaction{Execer: []byte(BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: Addr}
		return tx, nil
	} else {
		tx := &types.Transaction{Execer: []byte(paraName + BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: crypto.GetExecAddress(paraName + BrokerX)}
		return tx, nil
	}
}

func EmitInterchainSetEvent(){

}
